#!/usr/bin/env python3
import time

__author__ = 'intigua'

import subprocess
import os
import intiguautil
from intiguautil import Ssh

def rsync(src, dst):
    # rsync  -zrav  root@30.30.2.52:/usr/share/intigua /usr/share/
    cmd = ["rsync", "-zrav", "-e", "ssh -o \"BatchMode yes\"", src, dst]
    print("invoking rsync:", cmd)
    subprocess.check_call(cmd, stderr=subprocess.STDOUT)

def check_trust(dst):
    return 0 == Ssh(dst).run("ls")


class Backuper(object):
    def __init__(self, src, dst = None, dirs = None):
        if src == dst:
            raise Exception("Source and dest can't be the same!")
        self.dirs = dirs
        self.src = src
        self.dst = dst

    def file_exists(self,remote, path):
        if remote:
            return 0 == Ssh(remote).run(["ls" , path])
        else:
            return os.path.exists(path)

    def check_install_profile(self):
        return intiguautil.check_install_profile(self.src)

    def get_backup_dirs(self):
        dirs = ["/usr/share/intigua"]
        prof = self.check_install_profile()
        if prof == "intigua":
            return dirs
        return dirs + ["/usr/share/tomcat"]

    def try_backup_intigua(self):
        if not self.dirs:
            self.dirs = self.get_backup_dirs()
        print("Backup directores:", self.dirs)
        for d in self.dirs:
            # do not add trailing slash
            d = remove_trailing_slash(d)
            dstfolder = os.path.dirname(d)
            rsync(get_remote_path(self.src , d),get_remote_path(self.dst, dstfolder))

    def backup_intigua(self):
        try:
            self.try_backup_intigua()
        except Exception as e:
            print("rsync failed:", e)


def remove_trailing_slash(s):
    if s[-1] == "/":
        return s[:-1]
    return s

def get_remote_path(remote, path):
    if not remote:
        return path
    return remote + ":" + path

def start(args):

    from configparser import ConfigParser
    config = ConfigParser()
    config.read(args.conf)

    if args.configfile:
        if not config.has_section("Source"):
            config.add_section("Source")
        if not config.has_section("Destination"):
            config.add_section("Destination")

        if args.src:
            config.set("Source","address", args.src)
        else:
            config.remove_option("Source","address")

        if args.dst:
            config.set("Destination","address", args.dst)
        else:
            config.remove_option("Destination","address")

        with open(args.conf, 'wb') as configfile:
            config.write(configfile)
        return

    get_or_none = lambda section, configParam: config.get(section, configParam) if config.has_option(section, configParam) else None
    if not args.src:
        args.src = get_or_none("Source","address")
    if not args.dst:
        args.dst = get_or_none("Destination","address")

    if args.src:
        intiguautil.test_ssh(args.src)
    if args.dst:
        intiguautil.test_ssh(args.dst)

    b = Backuper(args.src, args.dst)

    if args.loop:
        # write a pid file:
        pid = str(os.getpid())

        while True:
            b.backup_intigua()
            time.sleep(5)
    else:
        b.backup_intigua()


def parse_args():
    import argparse   # only supported on python 2.7, rest of this script may be used with python 2.6
    parser = argparse.ArgumentParser(description='Intigua sync client')

    # Do not use the default paramter in the params that are also read from the config file!!
    parser.add_argument('--src', dest="src", type=str, help='Where to backup from. leave empty for config')
    parser.add_argument('--dst', dest="dst", type=str, help='Where to backup to. leave empty for config')
    parser.add_argument('--conf', dest='conf', type=str, default ="/etc/intigua/sync.conf",
                   help='Configuration file, to avoid settings the parameters in the command line every time. Parameters given on the command line override the configutration file. Default location is /etc/intigua/sync.conf')
    parser.add_argument('--createconfig',dest="configfile", action='store_true', help='Create the configuration at a location. does nothing else.')
    parser.add_argument('--pidfile', dest='pidfile', type=str, default=None, help='write a pid file and daemonize')

    parser.add_argument('--once', dest="loop", action='store_false', help='Run only once')

    args = parser.parse_args()
    return args

def main():
    args = parse_args()

    if args.pidfile:
        from daemonize import Daemonize
        daemon = Daemonize(app="intigua-sync", pid=args.pidfile, action=lambda : start(args))
        daemon.start()
    else:
        start(args)

if __name__ == "__main__":
    main()
